@@ -2,11 +2,11 @@
  * bluefish.c - the main function
  *
  * Copyright (C) 1998 Olivier Sessink and Chris Mazuc
- * Copyright (C) 1999-2009 Olivier Sessink
+ * Copyright (C) 1999-2011 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*#define DEBUG */
@@ -25,6 +24,15 @@
 #include <stdlib.h>				/* exit() on Solaris */
 #include <time.h>				/* nanosleep */
 
+#ifndef WIN32
+#include <signal.h>             /* sigaction */
+#endif
+
+#ifdef MAC_INTEGRATION
+#include <gtkosxapplication.h>
+#endif
+
+
 #include "bluefish.h"
 #include <libxml/parser.h>
 
@@ -38,13 +46,13 @@
 #endif
 #include "pixmap.h"				/* set_default_icon() */
 #include "bf_lib.h"				/* create_full_path() */
+#include "bfwin.h"
 #include "bookmark.h"			/* bmark_init() */
 #include "dialog_utils.h"		/* message_dialog_new() */
 #include "document.h"
 #include "filebrowser2.h"
 #include "file_dialogs.h"
 #include "gtk_easy.h"			/* flush_queue() */
-#include "gui.h"				/* gui_create_main() */
 #ifdef HAVE_MESSAGE_QUEUE
 #include "msg_queue.h"			/* msg_queue_start() */
 #else
@@ -57,6 +65,7 @@
 #include "filefilter.h"
 #include "file.h" /* file_handle() */
 #include "file_autosave.h"
+#include "languages.h"
 
 /*********************************************/
 /* this var is global for all bluefish files */
@@ -66,8 +75,14 @@ Tmain *main_v;
 /********************************/
 /* functions used in bluefish.c */
 /********************************/
+/*#define STARTUP_PROFILING*/
+
+#ifdef STARTUP_PROFILING
+static GTimer *startuptimer;
+#endif
+
 #ifndef __GNUC__
-void g_none(gchar * first, ...)
+void g_none(char * first, ...)
 {
 	return;
 }
@@ -81,15 +96,17 @@ void cb_print_version(const gchar * option_name, const gchar * value, gpointer d
 }
 
 static void init_default_session(Tsessionvars *session) {
-	session->view_main_toolbar = session->view_left_panel
+	session->enable_syntax_scan = session->view_main_toolbar
+			= session->view_left_panel
 			= session->filebrowser_focus_follow
 			= session->view_statusbar
 			= session->autoindent
 			= session->view_line_numbers
 			= session->view_cline
 			= session->view_blocks
+			= session->view_blockstack
 			= session->autocomplete
-			= session->view_mbhl
+			= session->show_mbhl
 			= 1;
 #ifdef HAVE_LIBENCHANT
 	session->spell_check_default = 1;
@@ -98,6 +115,31 @@ static void init_default_session(Tsessionvars *session) {
 	session->editor_tab_width = 3;
 }
 
+#ifndef WIN32
+static void sigterm_handler(int signalnum, siginfo_t *si, void *data) {
+	g_print("caught SIGTERM, exiting...\n");
+	bluefish_exit_request();
+}
+
+static void handle_signals(void) {
+	struct sigaction sa;
+	sa.sa_sigaction = sigterm_handler;
+	sigemptyset(&sa.sa_mask);
+	sa.sa_flags = SA_SIGINFO;
+	sigaction(SIGTERM, &sa, NULL);
+}
+#endif
+
+#ifdef MAC_INTEGRATION
+static gboolean osx_open_file_cb(GtkOSXApplication *app, gchar *path, gpointer user_data) {
+	GFile *file;
+	Tbfwin *bfwin = BFWIN(g_list_last(main_v->bfwinlist)->data);
+	g_print("osx_open_file_cb, open %s\n",path);
+	doc_new_from_input(bfwin,path,FALSE,FALSE,-1);
+	return TRUE;
+}
+#endif
+
 /*********************/
 /* the main function */
 /*********************/
@@ -105,24 +147,17 @@ static void init_default_session(Tsessionvars *session) {
 typedef struct {
 	Tbfwin *firstbfwin;
 	GList *filenames;
-#ifndef NOSPLASH
-	GtkWidget *splash_window;
-#endif							/* NOSPLASH */	
 	guint state;
 } Tstartup;
 
 static gboolean startup_in_idle(gpointer data) {
 	Tstartup *startup=data;
-	DEBUG_MSG("startup_in_idle, started state=%d\n",startup->state);
+	DEBUG_MSG("startup_in_idle, started state=%d, elapsed=\n",startup->state);
+#ifdef STARTUP_PROFILING
+	g_print("startup_in_idle, state=%d, elapsed=%d\n",startup->state,(gint)(g_timer_elapsed(startuptimer,NULL)*1000.0));
+#endif
 	switch (startup->state) {
 		case 0:
-#ifndef NOSPLASH
-			if (main_v->props.show_splash_screen) {
-				/* start splash screen somewhere here */
-				startup->splash_window = start_splash_screen();
-				splash_screen_set_label(_("starting up..."));
-			}
-#endif							/* NOSPLASH */
 			bluefish_load_plugins();
 			main_v->session = g_new0(Tsessionvars, 1);
 			init_default_session(main_v->session);
@@ -134,7 +169,7 @@ static gboolean startup_in_idle(gpointer data) {
 								  g_file_get_uri(uri));
 				g_object_unref(uri);
 			}
-			bftextview2_parse_static_colors();
+			bftextview2_init_globals();
 			langmgr_init();
 #ifdef HAVE_LIBENCHANT
 			bftextview2_spell_init();
@@ -142,7 +177,6 @@ static gboolean startup_in_idle(gpointer data) {
 			set_default_icon();
 			fb2config_init();			/* filebrowser2config */
 			filters_rebuild();
-			main_v->tooltips = gtk_tooltips_new();
 			main_v->bmarkdata = bookmark_data_new();
 			/* create the first window */
 			startup->firstbfwin = g_new0(Tbfwin, 1);
@@ -156,7 +190,7 @@ static gboolean startup_in_idle(gpointer data) {
 #endif							/* WITH_MSG_QUEUE */
 		break;
 		case 1:
-			gui_create_main(startup->firstbfwin);
+			bfwin_create_main(startup->firstbfwin);
 #ifdef WITH_MSG_QUEUE
 			if (main_v->props.open_in_running_bluefish) {
 				msg_queue_check_server(FALSE);
@@ -170,11 +204,12 @@ static gboolean startup_in_idle(gpointer data) {
 				DEBUG_MSG("startup_in_idle, we have filenames, load them\n");
 				startup->firstbfwin->focus_next_new_doc = TRUE;
 				while (tmplist) {
-					file_handle((GFile *)tmplist->data, startup->firstbfwin, NULL);
+					file_handle((GFile *)tmplist->data, startup->firstbfwin, NULL, TRUE);
 					tmplist = g_list_next(tmplist);
 				}
 			}
-			bmark_reload(startup->firstbfwin);
+			if (startup->firstbfwin->session == main_v->session)
+				bmark_reload(startup->firstbfwin); /* do not reload bookmarks for a project */
 			/* set GTK settings, must be AFTER the menu is created */
 			{
 				gchar *shortcutfilename;
@@ -192,46 +227,63 @@ static gboolean startup_in_idle(gpointer data) {
 		#endif
 		break;
 		case 3:
-			gui_show_main(startup->firstbfwin);
+			bfwin_show_main(startup->firstbfwin);
+#ifdef MAC_INTEGRATION
+			{
+				GtkOSXApplication *theApp = g_object_new(GTK_TYPE_OSX_APPLICATION, NULL);
+				gtk_osxapplication_ready(theApp);
+				g_signal_connect(theApp, "NSApplicationOpenFile", osx_open_file_cb, NULL);
+			}
+#endif
 		break;
 		case 4:
+			main_v->recentm = gtk_recent_manager_get_default();
 			doc_scroll_to_cursor(BFWIN(startup->firstbfwin)->current_document);
 			modified_on_disk_check_init();
-#ifndef NOSPLASH
-			if (main_v->props.show_splash_screen) {
-				gtk_widget_destroy(startup->splash_window);
-			}
-#endif							/* NOSPLASH */
+#ifndef WIN32            
+			handle_signals();
+#endif            
 			g_free(startup);
 			return FALSE;
 		break;
 	}
+#ifdef STARTUP_PROFILING
+	g_print("startup_in_idle, end of state=%d, elapsed=%d\n",startup->state,(gint)(g_timer_elapsed(startuptimer,NULL)*1000.0));
+#endif
 	startup->state++;
 	return TRUE;	
 }
 
 int main(int argc, char *argv[])
 {
-	gboolean arg_curwindow = FALSE, arg_newwindow=FALSE;
-	gchar **files = NULL;
+	static gboolean arg_curwindow = FALSE, arg_newwindow=FALSE;
+	static gchar **files = NULL;
 	Tstartup *startup;
 #ifdef WIN32
  	gchar *path;
- #endif
+#endif
 	GError *error = NULL;
 	GOptionContext *context;
 	const GOptionEntry options[] = {
-		{"curwindow", 'c', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_NONE, &arg_curwindow,
+		{"curwindow", 'c', 0, G_OPTION_ARG_NONE, &arg_curwindow,
 		 N_("Open in existing window."), NULL},
-		{"newwindow", 'n', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_NONE, &arg_newwindow,
+		{"newwindow", 'n', 0, G_OPTION_ARG_NONE, &arg_newwindow,
 		 N_("Open in new window."), NULL},
 		{"version", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, (void *) cb_print_version,
 		 N_("Print version information."), NULL},
-		{G_OPTION_REMAINING, 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_FILENAME_ARRAY, &files,
+		{G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &files,
 		 "Special option that collects any remaining arguments for us", NULL},
 		{NULL}
 	};
 
+	if (!g_thread_supported())
+		g_thread_init(NULL);
+/*  gdk_threads_init ();*/
+
+#ifdef STARTUP_PROFILING
+	startuptimer = g_timer_new();
+#endif /* STARTUP_PROFILING */
+
 #ifdef ENABLE_NLS
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
@@ -240,9 +292,6 @@ int main(int argc, char *argv[])
 	textdomain(PACKAGE);
 #endif							/* ENABLE_NLS */
 
-	if (!g_thread_supported())
-		g_thread_init(NULL);
-/*  gdk_threads_init ();*/
 	gtk_rc_parse_string ("style \"bluefish-small-close-button-style\"\n"
                        "{\n"
                           "GtkWidget::focus-padding = 0\n"
@@ -250,8 +299,8 @@ int main(int argc, char *argv[])
                           "xthickness = 0\n"
                           "ythickness = 0\n"
                        "}\n"
-                       "widget \"*.bluefish-small-close-button\" style \"bluefish-small-close-button-style\"");
-
+                       "widget \"*.bluefish-small-close-button\" style \"bluefish-small-close-button-style\""
+                       );
 	context = g_option_context_new(_(" [FILE(S)]"));
 #ifdef ENABLE_NLS
 	g_option_context_add_main_entries(context, options, PACKAGE);
@@ -309,11 +358,27 @@ int main(int argc, char *argv[])
 	g_free(path);
 #endif
 
+#ifdef DEBUG_PATHS
+	{
+		gchar *cwd = g_get_current_dir();
+		g_print("current directory=%s\n",cwd);
+		g_free(cwd);
+	}
+#endif
 	rcfile_check_directory();
 	rcfile_parse_main();
+#ifdef ENABLE_NLS
 	if (main_v->props.language!=NULL && main_v->props.language[0]!='\0') {
+#ifndef WIN32
 		setlocale(LC_ALL, main_v->props.language);
+#else /* WIN32 */
+		if (!lingua_set_thread_locale_on_windows(main_v->props.language)) {
+			g_free(main_v->props.language);
+			main_v->props.language = NULL;
+		}
+#endif /* WIN32 */
 	}
+#endif /* ENABLE_NLS */
 	if (main_v->props.open_in_running_bluefish) {
 #ifdef WITH_MSG_QUEUE
 		/* start message queue as early as possible so a running bluefish process has a lot of 
@@ -321,17 +386,21 @@ int main(int argc, char *argv[])
 		msg_queue_start(startup->filenames, (arg_newwindow || (main_v->props.open_in_new_window && !arg_curwindow) ) );
 #else
 		if (!ipc_bf2bf_start(startup->filenames, (arg_newwindow || (main_v->props.open_in_new_window && !arg_curwindow) ) )) {
+			gdk_notify_startup_complete();
 			exit(0);
 		}
 #endif /* WITH_MSG_QUEUE */
 	}
-
+#ifdef MAC_INTEGRATION
+	{
+	GtkOSXApplication *theApp = g_object_new(GTK_TYPE_OSX_APPLICATION, NULL);
+	}
+#endif
 	g_idle_add_full(G_PRIORITY_DEFAULT_IDLE-50, startup_in_idle, startup, NULL);
 	DEBUG_MSG("main, before gtk_main()\n");
-/*  gdk_threads_enter ();*/
 	gtk_main();
-/*  gdk_threads_leave ();*/
 	DEBUG_MSG("main, after gtk_main()\n");
+
 	autosave_cleanup();
 #ifdef WITH_MSG_QUEUE
 	/* do the cleanup */
@@ -339,11 +408,20 @@ int main(int argc, char *argv[])
 #else
 	ipc_bf2bf_cleanup();
 #endif /* WITH_MSG_QUEUE */
+
+#ifdef MEMORY_LEAK_DEBUG
 	DEBUG_MSG("calling fb2config_cleanup()\n");
 	fb2config_cleanup();
+	main_v->bmarkdata = bookmark_data_cleanup(main_v->bmarkdata);
 	langmgr_cleanup();
 	xmlCleanupParser();
+	
+	/*cairo_debug_reset_static_data();
+	FcFini();*/ 
 	DEBUG_MSG("Bluefish: exiting cleanly\n");
+#else
+	exit(0);
+#endif
 	return 0;
 }
 
@@ -359,11 +437,9 @@ void bluefish_exit_request()
 	g_list_free(tmplist);
 	tmplist = g_list_first(main_v->bfwinlist);
 	while (tmplist) {
-		main_window_delete_event_lcb(NULL,NULL,BFWIN(tmplist->data));
-/*		if (tmpb) {
-			file_close_all_cb(NULL, BFWIN(tmplist->data));
-		}*/
+		Tbfwin *bfwin = BFWIN(tmplist->data);
 		tmplist = g_list_next(tmplist);
+		bfwin_delete_event(NULL,NULL,bfwin);
 	}
 	/* if we still have modified documents we don't do a thing,
 	   if we don't have them we can quit */
@@ -381,6 +457,7 @@ void bluefish_exit_request()
 		gtk_widget_hide(GTK_WIDGET(tmplist->data));
 		tmplist = g_list_next(tmplist);
 	}
+	g_list_free(tmplist);
 	flush_queue();
 
 	rcfile_save_global_session();
