@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * rcfile.c - loading and parsing of the configfiles
  *
- * Copyright (C) 2000-2010 Olivier Sessink
+ * Copyright (C) 2000-2011 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -95,19 +95,22 @@ typedef struct {
 
 static GHashTable *main_configlist = NULL;
 
-static void free_configlist(GHashTable * configlist)
+static void
+free_configlist(GHashTable * configlist)
 {
 	g_hash_table_destroy(configlist);
 }
 
-static void free_config_list_item(gpointer data)
+static void
+free_config_list_item(gpointer data)
 {
 	g_slice_free(Tconfig_list_item, data);
 }
 
 /*this should add 1 empty entry to the configuration list */
-GHashTable *make_config_list_item(GHashTable * config_list, void *pointer_to_var, unsigned char type_of_var,
-								  gchar * name_of_var, gint len)
+GHashTable *
+make_config_list_item(GHashTable * config_list, void *pointer_to_var, unsigned char type_of_var,
+					  gchar * name_of_var, gint len)
 {
 	Tconfig_list_item *config_list_item;
 	if (!pointer_to_var) {
@@ -123,16 +126,18 @@ GHashTable *make_config_list_item(GHashTable * config_list, void *pointer_to_var
 	return config_list;
 }
 
-static void init_prop_integer(GHashTable ** config_list, void *pointer_to_var, gchar * name_of_var,
-							  gint default_value, gboolean set_default)
+static void
+init_prop_integer(GHashTable ** config_list, void *pointer_to_var, gchar * name_of_var,
+				  gint default_value, gboolean set_default)
 {
 	*config_list = make_config_list_item(*config_list, pointer_to_var, 'i', name_of_var, 0);
 	if (set_default)
 		*(gint *) pointer_to_var = default_value;
 }
 
-static void init_prop_string(GHashTable ** config_list, void *pointer_to_var, gchar * name_of_var,
-							 const gchar * default_value)
+static void
+init_prop_string(GHashTable ** config_list, void *pointer_to_var, gchar * name_of_var,
+				 const gchar * default_value)
 {
 	*config_list = make_config_list_item(*config_list, pointer_to_var, 's', name_of_var, 0);
 	if (*(gchar **) pointer_to_var == NULL) {
@@ -142,8 +147,9 @@ static void init_prop_string(GHashTable ** config_list, void *pointer_to_var, gc
 			  default_value, *(gchar **) pointer_to_var);
 }
 
-static void init_prop_string_with_escape(GHashTable ** config_list, void *pointer_to_var, gchar * name_of_var,
-										 gchar * default_value)
+static void
+init_prop_string_with_escape(GHashTable ** config_list, void *pointer_to_var, gchar * name_of_var,
+							 gchar * default_value)
 {
 	*config_list = make_config_list_item(*config_list, pointer_to_var, 'e', name_of_var, 0);
 	if (*(gchar **) pointer_to_var == NULL && default_value) {
@@ -152,8 +158,8 @@ static void init_prop_string_with_escape(GHashTable ** config_list, void *pointe
 	DEBUG_MSG("init_prop_string, name_of_var=%s, default_value=%s\n", name_of_var, default_value);
 }
 
-static void init_prop_stringlist(GHashTable ** config_list, void *pointer_to_var, gchar * name_of_var,
-								 gboolean setNULL)
+static void
+init_prop_stringlist(GHashTable ** config_list, void *pointer_to_var, gchar * name_of_var, gboolean setNULL)
 {
 	*config_list = make_config_list_item(*config_list, pointer_to_var, 'l', name_of_var, 0);
 	if (setNULL) {
@@ -161,8 +167,9 @@ static void init_prop_stringlist(GHashTable ** config_list, void *pointer_to_var
 	}
 }
 
-static void init_prop_arraylist(GHashTable ** config_list, void *pointer_to_var, gchar * name_of_var,
-								gint len, gboolean setNULL)
+static void
+init_prop_arraylist(GHashTable ** config_list, void *pointer_to_var, gchar * name_of_var,
+					gint len, gboolean setNULL)
 {
 	*config_list = make_config_list_item(*config_list, pointer_to_var, 'a', name_of_var, len);
 	if (setNULL) {
@@ -171,8 +178,9 @@ static void init_prop_arraylist(GHashTable ** config_list, void *pointer_to_var,
 }
 
 /* limited lists should have the most recent on top, the last entries will be cut if there are too many entries */
-static void init_prop_limitedstringlist(GHashTable ** config_list, void *pointer_to_var, gchar * name_of_var,
-										gint len, gboolean setNULL)
+static void
+init_prop_limitedstringlist(GHashTable ** config_list, void *pointer_to_var, gchar * name_of_var,
+							gint len, gboolean setNULL)
 {
 	*config_list = make_config_list_item(*config_list, pointer_to_var, 'm', name_of_var, len);
 	if (setNULL) {
@@ -180,7 +188,8 @@ static void init_prop_limitedstringlist(GHashTable ** config_list, void *pointer
 	}
 }
 
-static gint save_config_file(GHashTable * config_list, GFile * file)
+static gint
+save_config_file(GHashTable * config_list, GFile * file)
 {
 	gchar *tmpstring = NULL, *tmpstring2;
 	GList *rclist, *tmplist2;
@@ -287,7 +296,8 @@ static gint save_config_file(GHashTable * config_list, GFile * file)
 	return retval;
 }
 
-static gboolean parse_config_file(GHashTable * config_list, GFile * file)
+static gboolean
+parse_config_file(GHashTable * config_list, GFile * file)
 {
 	gboolean retval = FALSE;
 	gchar *tmpstring = NULL, *tmpstring2;
@@ -391,12 +401,14 @@ static gboolean parse_config_file(GHashTable * config_list, GFile * file)
 	return retval;
 }
 
-static GHashTable *props_init_main(GHashTable * config_rc)
+static GHashTable *
+props_init_main(GHashTable * config_rc)
 {
 	init_prop_integer(&config_rc, &main_v->props.do_periodic_check, "do_periodic_check:", 1, TRUE);
 	init_prop_string(&config_rc, &main_v->props.editor_font_string, "editor_font_string:", "monospace 10");
 	init_prop_integer(&config_rc, &main_v->props.editor_smart_cursor, "editor_smart_cursor:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.editor_tab_indent_sel, "editor_tab_indent_sel:", 0, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.editor_auto_close_brackets, "editor_auto_close_brackets:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.use_system_tab_font, "use_system_tab_font:", 1, TRUE);
 	init_prop_string(&config_rc, &main_v->props.tab_font_string, "tab_font_string:", "");
 	init_prop_string(&config_rc, &main_v->props.tab_color_modified, "tab_color_modified:", "#0000FF");
@@ -441,6 +453,7 @@ static GHashTable *props_init_main(GHashTable * config_rc)
 					  TRUE);
 	init_prop_integer(&config_rc, &main_v->props.encoding_search_Nbytes, "encoding_search_Nbytes:", 500,
 					  TRUE);
+	init_prop_integer(&config_rc, &main_v->props.encoding_search_Nbytes, "max_window_title:", 0,TRUE);
 	init_prop_integer(&config_rc, &main_v->props.document_tabposition, "document_tabposition:",
 					  (gint) GTK_POS_BOTTOM, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.leftpanel_tabposition, "leftpanel_tabposition:",
@@ -482,7 +495,8 @@ static GHashTable *props_init_main(GHashTable * config_rc)
 	init_prop_integer(&config_rc, &main_v->props.block_folding_mode, "block_folding_mode:", 1, TRUE);
 	init_prop_arraylist(&config_rc, &main_v->props.highlight_styles, "highlight_styles:", 3, TRUE);
 	init_prop_arraylist(&config_rc, &main_v->props.bflang_options, "bflang_options:", 3, TRUE);
-	init_prop_string(&config_rc, &main_v->props.autocomp_accel_string, "autocomp_accel_string:", "<Control>space");
+	init_prop_string(&config_rc, &main_v->props.autocomp_accel_string, "autocomp_accel_string:",
+					 "<Control>space");
 	init_prop_integer(&config_rc, &main_v->props.load_reference, "load_reference:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.show_autocomp_reference, "show_autocomp_reference:", 1,
 					  TRUE);
@@ -490,7 +504,7 @@ static GHashTable *props_init_main(GHashTable * config_rc)
 	init_prop_integer(&config_rc, &main_v->props.delay_full_scan, "delay_full_scan:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.delay_scan_time, "delay_scan_time:", 900, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.autocomp_popup_mode, "autocomp_popup_mode:", 1, TRUE);
-	init_prop_integer(&config_rc, &main_v->props.reduced_scan_triggers, "reduced_scan_triggers:", 1, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.reduced_scan_triggers, "reduce_scan_triggers:", 0, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.autosave, "autosave:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.autosave_time, "autosave_time:", 60, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.autosave_location_mode, "autosave_location_mode:", 0, TRUE);
@@ -501,12 +515,12 @@ static GHashTable *props_init_main(GHashTable * config_rc)
 	return config_rc;
 }
 
-static gboolean merge_config_files(GFile * oldrc, GFile * oldsession, GFile * newrc, GFile * newsession)
+static gboolean
+merge_config_files(GFile * oldrc, GFile * oldsession, GFile * newrc, GFile * newsession)
 {
 	GError *gerror = NULL;
 	GInputStream *istream;
 	GOutputStream *ostream;
-	gssize size;
 
 	istream = (GInputStream *) g_file_read(oldrc, NULL, &gerror);
 	if (gerror) {
@@ -521,7 +535,7 @@ static gboolean merge_config_files(GFile * oldrc, GFile * oldsession, GFile * ne
 		g_input_stream_close(istream, NULL, &gerror);
 		return FALSE;
 	}
-	size = g_output_stream_splice(ostream, istream, G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE, NULL, &gerror);
+	g_output_stream_splice(ostream, istream, G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE, NULL, &gerror);
 	if (gerror) {
 		g_print("config file migration error %d:%s", gerror->code, gerror->message);
 		g_error_free(gerror);
@@ -535,7 +549,7 @@ static gboolean merge_config_files(GFile * oldrc, GFile * oldsession, GFile * ne
 		g_output_stream_close(ostream, NULL, &gerror);
 		return FALSE;
 	}
-	size = g_output_stream_splice(ostream, istream, G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE, NULL, &gerror);
+	g_output_stream_splice(ostream, istream, G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE, NULL, &gerror);
 	if (gerror) {
 		g_print("config file migration error %d:%s", gerror->code, gerror->message);
 		g_error_free(gerror);
@@ -552,7 +566,8 @@ static gboolean merge_config_files(GFile * oldrc, GFile * oldsession, GFile * ne
 	return TRUE;
 }
 
-static void migrate_config_files(GHashTable * main_configlist, GFile * newrc)
+static void
+migrate_config_files(GHashTable * main_configlist, GFile * newrc)
 {
 	GFile *oldrc, *oldsession, *newsession;
 	oldrc = user_bfdir(OLDCONFIG);
@@ -567,7 +582,8 @@ static void migrate_config_files(GHashTable * main_configlist, GFile * newrc)
 	}
 }
 
-void rcfile_parse_main(void)
+void
+rcfile_parse_main(void)
 {
 	GFile *file;
 
@@ -582,7 +598,7 @@ void rcfile_parse_main(void)
 
 	file = user_bfdir(CURCONFIG);
 	if (!parse_config_file(main_configlist, file)) {
-		g_warning("no configfile %s, try to convert config files from older versions\n",CURCONFIG);
+		g_warning("no configfile %s, try to convert config files from older versions\n", CURCONFIG);
 		/* probably there is no configfile. try to migrate the configfile from a previous 
 		   version */
 		migrate_config_files(main_configlist, file);
@@ -622,8 +638,8 @@ void rcfile_parse_main(void)
 		main_v->props.external_command =
 			g_list_append(main_v->props.external_command,
 						  array_from_arglist(_("Windows XP Firefox"),
-							  	  	  	     "\"C:\\Program Files\\Mozilla Firefox\\firefox.exe\" '%p'",
-							  	  	  	     "1", NULL));
+											 "\"C:\\Program Files\\Mozilla Firefox\\firefox.exe\" '%p'",
+											 "1", NULL));
 		main_v->props.external_command =
 			g_list_append(main_v->props.external_command,
 						  array_from_arglist(_("Windows XP Internet Explorer"),
@@ -738,6 +754,14 @@ void rcfile_parse_main(void)
 			g_list_append(main_v->props.external_outputbox,
 						  array_from_arglist(_("php"), "(.*) in (/[a-zA-Z0-9/_.-]+) on line ([0-9]+)", "2",
 											 "3", "1", "php '%f'|", NULL));
+		main_v->props.external_outputbox =
+			g_list_append(main_v->props.external_outputbox,
+						  array_from_arglist(_("php syntax check"), "(.*) in (/[a-zA-Z0-9/_.-]+) on line ([0-9]+)", "2",
+											 "3", "1", "php -l -q -f '%f'|", NULL));
+		main_v->props.external_outputbox =
+			g_list_append(main_v->props.external_outputbox,
+							array_from_arglist(_("perl syntax check"), "(.*) at (/[a-zA-Z0-9/_.-]+) line ([0-9]+)", "2",
+											 "3", "1", "perl -c '%f'|", NULL));
 	}
 	if (main_v->props.external_filter == NULL) {
 		main_v->props.external_filter =
@@ -793,7 +817,7 @@ void rcfile_parse_main(void)
 						  array_from_arglist(_("Links2 (graphics)"), "links2 -g '%p'&", "0", NULL));
 		main_v->props.external_command =
 			g_list_append(main_v->props.external_command,
-						  array_from_arglist(_("chmod a+x"), "chmod a+x %s", "0", NULL));
+						  array_from_arglist(_("chmod a+x"), "chmod a+x %f", "0", NULL));
 	}
 #endif							/* ifdef WIN32 elif PLATFORM_DARWIN */
 	if (main_v->props.templates == NULL) {
@@ -812,10 +836,12 @@ void rcfile_parse_main(void)
 		main_v->props.templates =
 			g_list_append(main_v->props.templates,
 						  array_from_arglist(_("PHP"), PKGDATADIR "/templates/PHP", NULL));
+		/* TODO: list the templates in the directory */
 	}
 }
 
-gint rcfile_save_main(void)
+gint
+rcfile_save_main(void)
 {
 	gint ret;
 	GFile *filename = user_bfdir(CURCONFIG);
@@ -827,7 +853,8 @@ gint rcfile_save_main(void)
 #ifndef WIN32
 #define DIR_MODE (S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)	/* same as 0755 */
 #endif
-void rcfile_check_directory(void)
+void
+rcfile_check_directory(void)
 {
 	gchar *rcdir = g_strconcat(g_get_home_dir(), "/." PACKAGE, NULL);
 	DEBUG_PATH("test if rcfile directory %s exists\n", rcdir);
@@ -841,24 +868,16 @@ void rcfile_check_directory(void)
 	g_free(rcdir);
 }
 
-void rcfile_save_configfile_menu_cb(gpointer callback_data, guint action, GtkWidget * widget)
+void
+rcfile_save_accelerators(void)
 {
-	switch (action) {
-	case 0:
-		{
-			gchar *shortcutfilename = g_strconcat(g_get_home_dir(), "/." PACKAGE "/menudump_2", NULL);
-			gtk_accel_map_save(shortcutfilename);
-			g_free(shortcutfilename);
-		}
-		break;
-	default:
-		DEBUG_MSG_C("rcfile_save_configfile_menu_cb, unknown action %d\n", action);
-		g_return_if_reached();
-		break;
-	}
+	gchar *shortcutfilename = g_strconcat(g_get_home_dir(), "/." PACKAGE "/menudump_2", NULL);
+	gtk_accel_map_save(shortcutfilename);
+	g_free(shortcutfilename);
 }
 
-static GHashTable *return_globalsession_configlist(gboolean init_values)
+static GHashTable *
+return_globalsession_configlist(gboolean init_values)
 {
 	GHashTable *config_rc = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, free_config_list_item);
 	init_prop_integer(&config_rc, &main_v->globses.main_window_h, "main_window_height:", 400, init_values);
@@ -895,21 +914,32 @@ static GHashTable *return_globalsession_configlist(gboolean init_values)
 	return config_rc;
 }
 
-static GHashTable *return_session_configlist(GHashTable * configlist, Tsessionvars * session)
+static GHashTable *
+return_session_configlist(GHashTable * configlist, Tsessionvars * session)
 {
 	/* this function should *NOT* initialize any values to default values
 	   because it is also used on existing sessions that already have a value, and
 	   that would wipe out the value of the existing session */
+	init_prop_integer(&configlist, &session->enable_syntax_scan, "enable_syntax_scan:", 1, FALSE);
 	init_prop_integer(&configlist, &session->wrap_text_default, "wrap_text_default:", 1, FALSE);
 	init_prop_integer(&configlist, &session->autoindent, "autoindent:", 1, FALSE);
 	init_prop_integer(&configlist, &session->editor_tab_width, "editor_tab_width:", 3, FALSE);
-	init_prop_integer(&configlist, &session->editor_indent_wspaces, "editor_indent_wspaces:", 0, TRUE);
+	init_prop_integer(&configlist, &session->editor_indent_wspaces, "editor_indent_wspaces:", 0, FALSE);
 	init_prop_integer(&configlist, &session->view_line_numbers, "view_line_numbers:", 1, FALSE);
 	init_prop_integer(&configlist, &session->view_cline, "view_cline:", 1, FALSE);
 	init_prop_integer(&configlist, &session->view_blocks, "view_blocks:", 1, FALSE);
+	init_prop_integer(&configlist, &session->view_blockstack, "view_blockstack:", 1, FALSE);
 	init_prop_integer(&configlist, &session->autocomplete, "autocomplete:", 1, FALSE);
 	init_prop_integer(&configlist, &session->show_mbhl, "show_mbhl:", 1, FALSE);
-	init_prop_integer(&configlist, &session->display_right_margin, "display_right_margin:", 0, TRUE);
+	
+	init_prop_integer(&configlist, &session->snr3_type, "snr_type:", 1, 0);
+	init_prop_integer(&configlist, &session->snr3_replacetype, "snr_replacetype:", 1, 0);
+	init_prop_integer(&configlist, &session->snr3_scope, "snr_scope:", 1, 0);
+	init_prop_integer(&configlist, &session->snr3_casesens, "snr_casesens:", 1, 0);
+	init_prop_integer(&configlist, &session->snr3_escape_chars, "snr_escape_chars:", 1, 0);
+	init_prop_integer(&configlist, &session->snr3_dotmatchall, "snr_dotmatchall:", 1, 1);
+
+	init_prop_integer(&configlist, &session->display_right_margin, "display_right_margin:", 0, FALSE);
 #ifdef HAVE_LIBENCHANT
 	init_prop_integer(&configlist, &session->spell_check_default, "spell_check_default:", 1, FALSE);
 	init_prop_integer(&configlist, &session->spell_insert_entities, "spell_insert_entities:", 0, FALSE);
@@ -918,8 +948,9 @@ static GHashTable *return_session_configlist(GHashTable * configlist, Tsessionva
 	init_prop_string(&configlist, &session->template, "template:", NULL);
 	init_prop_string_with_escape(&configlist, &session->webroot, "webroot:", NULL);
 	init_prop_string_with_escape(&configlist, &session->documentroot, "documentroot:", NULL);
-	init_prop_limitedstringlist(&configlist, &session->searchlist, "searchlist:", 10, FALSE);
-	init_prop_limitedstringlist(&configlist, &session->replacelist, "replacelist:", 10, FALSE);
+	init_prop_limitedstringlist(&configlist, &session->searchlist, "searchlist:", 15, FALSE);
+	init_prop_limitedstringlist(&configlist, &session->replacelist, "replacelist:", 15, FALSE);
+	init_prop_limitedstringlist(&configlist, &session->filegloblist, "filegloblist:", 15, FALSE);
 	init_prop_stringlist(&configlist, &session->classlist, "classlist:", FALSE);
 	init_prop_stringlist(&configlist, &session->colorlist, "colorlist:", FALSE);
 	init_prop_stringlist(&configlist, &session->targetlist, "targetlist:", FALSE);
@@ -964,13 +995,33 @@ static GHashTable *return_session_configlist(GHashTable * configlist, Tsessionva
 	return configlist;
 }
 
-static void setup_session_after_parse(Tsessionvars * session)
+static void
+setup_session_after_parse(Tsessionvars * session)
 {
 	if (session->editor_tab_width < 2)
 		session->editor_tab_width = 3;
 
 	if (session->default_mime_type == NULL)
 		session->default_mime_type = g_strdup("text/plain");
+	
+	if (session->filegloblist == NULL) {
+		session->filegloblist = list_from_arglist(TRUE, "*", 
+				"*.c",
+				"*.cgi",
+				"*.cpp",
+				"*.css",
+				"*.h",
+				"*.html",
+				"*.htm",
+				"*.java",
+				"*.js",
+				"*.php",
+				"*.pl",
+				"*.py",
+				"*.shtml",
+				"*.txt",
+				"*.xml" , NULL);
+	}
 /* TODO: set spell check language to a sensible default
 	 
 #ifdef HAVE_LIBENCHANT
@@ -980,7 +1031,8 @@ static void setup_session_after_parse(Tsessionvars * session)
 */
 }
 
-static GHashTable *return_project_configlist(Tproject * project)
+static GHashTable *
+return_project_configlist(Tproject * project)
 {
 	GHashTable *configlist = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, free_config_list_item);
 	init_prop_string(&configlist, &project->name, "name:", _("Untitled Project"));
@@ -990,7 +1042,8 @@ static GHashTable *return_project_configlist(Tproject * project)
 	return configlist;
 }
 
-gboolean rcfile_parse_project(Tproject * project, GFile * file)
+gboolean
+rcfile_parse_project(Tproject * project, GFile * file)
 {
 	gboolean retval;
 	GHashTable *configlist = return_project_configlist(project);
@@ -1000,7 +1053,8 @@ gboolean rcfile_parse_project(Tproject * project, GFile * file)
 	return retval;
 }
 
-gboolean rcfile_save_project(Tproject * project, GFile * file)
+gboolean
+rcfile_save_project(Tproject * project, GFile * file)
 {
 	gboolean retval;
 	GHashTable *configlist = return_project_configlist(project);
@@ -1014,7 +1068,8 @@ gboolean rcfile_save_project(Tproject * project, GFile * file)
 	return retval;
 }
 
-gboolean rcfile_save_global_session(void)
+gboolean
+rcfile_save_global_session(void)
 {
 	gboolean retval;
 	GFile *filename;
@@ -1035,7 +1090,8 @@ gboolean rcfile_save_global_session(void)
 
 /* should be called AFTER the normal properties are loaded, becauses return_session_configlist() uses
  settings from main_v->props */
-gboolean rcfile_parse_global_session(void)
+gboolean
+rcfile_parse_global_session(void)
 {
 	gboolean retval;
 	GFile *file;
@@ -1054,7 +1110,7 @@ gboolean rcfile_parse_global_session(void)
 							   "image/jpeg:text/x-install:text/x-readme:text/x-chdr:text/x-csrc:image/png:text/x-authors:text/x-copying:text/x-makefile",
 							   "", NULL);
 		main_v->globses.filefilters = g_list_prepend(main_v->globses.filefilters, arr);
-		arr = array_from_arglist(_("Web files"), "1", "text/html:image/png:image/jpeg", "", NULL);
+		arr = array_from_arglist(_("Web files"), "1", "text/html:text/css:image/png:image/jpeg", "", NULL);
 		main_v->globses.filefilters = g_list_prepend(main_v->globses.filefilters, arr);
 		/*arr = array_from_arglist(_("Java programming"),"1", "java:image:jsp", NULL);
 		   main_v->globses.filefilters = g_list_append(main_v->globses.filefilters, arr); */
