@@ -1,11 +1,11 @@
 /* Bluefish HTML Editor
  * stringlist.c - functions that deal with stringlists
  *
- * Copyright (C) 1999-2002 Olivier Sessink
+ * Copyright (C) 1999-2010 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* #define DEBUG */
@@ -48,30 +47,6 @@
 /************************************************************************/
 /************************************************************************/
 
-/* DEPRECATED BY g_strv_length
- * count_array:
- * @array: #gchar** with the NULL terminated array to count
- *
- * counts the number of entries in a NULL terminated array
- *
- * Return value: #gint with number of entries
- * /
-gint count_array(gchar **array) {
-	gint count=0;
-	gchar **tmpchar=array;
-	
-	if (!tmpchar) {
-		DEBUG_MSG("count_array, no array!?!?\n");
-		return 0;
-	}
-
-	while (*tmpchar != NULL) {
-		count++;
-		tmpchar++;
-	}
-	return count;
-}
-*/
 /**
  * array_to_string:
  * @array: #gchar** with NULL terminated array
@@ -85,29 +60,32 @@ gint count_array(gchar **array) {
  * Return value: #gchar* newly allocated string
  */
 gchar *array_to_string(gchar **array) {
-	g_return_val_if_fail(array,g_strdup(""));
 	gchar **tmp, *escaped1, *finalstring;
 	gint newsize=1;
-	DEBUG_MSG("array_to_string, started\n");
-	finalstring = g_malloc0(newsize);
+	gsize allocsize=64;
+	g_return_val_if_fail(array,g_strdup(""));
+	finalstring = g_malloc0(allocsize);
 	tmp = array;
 	while(*tmp) {
-		DEBUG_MSG("array_to_string, *tmp = %s\n", *tmp);
+		/*DEBUG_MSG("array_to_string, *tmp = %s\n", *tmp);*/
 		escaped1 = escape_string(*tmp, TRUE);
 		newsize += strlen(escaped1)+1;
-		finalstring = g_realloc(finalstring, newsize);
+		if (newsize > allocsize) {
+			allocsize = MAX(newsize, (allocsize * 2));
+			finalstring = g_realloc(finalstring, allocsize);
+		}
 		strcat(finalstring, escaped1);
 		finalstring[newsize-2] = ':';
 		finalstring[newsize-1] = '\0';
 		g_free(escaped1);
 		tmp++;
-	}	
-	DEBUG_MSG("array_to_string, finalstring = %s\n", finalstring);
+	}
+	DEBUG_MSG("array_to_string, len(finalstring) = %zd\n", strlen(finalstring));
 	return finalstring;
 }
 
-#define ARRAYBLOCKSIZE 6
-#define BUFBLOCKSIZE 60
+#define ARRAYBLOCKSIZE 8
+#define BUFBLOCKSIZE 64
 /**
  * string_to_array:
  * @string: #gchar* with the string to convert
@@ -257,7 +235,7 @@ GList *list_from_arglist(gboolean allocate_strings, ...) {
 	va_start(args, allocate_strings);
 	s = va_arg(args, gchar*);
 	while (s) {
-		retval = g_list_append(retval, s);
+		retval = g_list_append(retval, allocate_strings ? g_strdup(s) : s);
 		s = va_arg (args, gchar*);
 	}
 	va_end (args);
@@ -293,7 +271,7 @@ gint free_stringlist(GList * which_list)
 
 	tmplist = g_list_first(which_list);
 	while (tmplist != NULL) {
-		DEBUG_MSG("free_stringlist, tmplist->data(%p)\n", tmplist->data);
+		/*DEBUG_MSG("free_stringlist, tmplist->data(%p)\n", tmplist->data);*/
 		g_free(tmplist->data);
 		tmplist = g_list_next(tmplist);
 	}
@@ -335,17 +313,6 @@ gint free_arraylist(GList * which_list)
 	which_list = NULL;
 	return 1;
 }
-/* deprecated by g_strdupv
-gchar **duplicate_stringarray(gchar **array) {
-	gchar **newchar;
-	gint i;
-
-	newchar = g_malloc0((count_array(array)+1)*sizeof(gchar *));
-	for (i=0; array[i] != NULL ; i++) {
-		newchar[i] = g_strdup(array[i]);
-	}
-	return newchar;
-}*/
 
 GList *duplicate_arraylist(GList *arraylist) {
 	GList *tmplist;
@@ -353,7 +320,7 @@ GList *duplicate_arraylist(GList *arraylist) {
 
 	tmplist = g_list_first(arraylist);
 	while (tmplist != NULL) {
-		newlist = g_list_append(newlist, duplicate_stringarray((gchar **)tmplist->data));
+		newlist = g_list_append(newlist, g_strdupv((gchar **)tmplist->data));
 		tmplist = g_list_next(tmplist);
 	}
 	return newlist;
@@ -456,7 +423,7 @@ GList *get_stringlist(GFile * filename, GList * which_list) {
  * if maxentries is 0 all entries will be stored
  *
  * Return value: #gboolean TRUE on success, FALSE on failure
- */
+ * /
 gboolean put_stringlist_limited(GFile * file, GList * which_list, gint maxentries, gboolean is_arraylist) {
 	GString *strbuffer;
 	GError *error=NULL;
@@ -495,13 +462,63 @@ gboolean put_stringlist_limited(GFile * file, GList * which_list, gint maxentrie
 	g_string_free(strbuffer,TRUE);
 	DEBUG_MSG("put_stringlist_limited, finished, filedescriptor closed\n");
 	return TRUE;
+}*/
+
+/**
+ * stringlist_to_string:
+ * @stringlist: a #GList * to convert
+ * @delimiter: a #const gchar * item with the delimiter
+ * 
+ * this function will convert a stringlist (GList that contains 
+ * only \0 terminated gchar* elements) to a string, putting the 
+ * delimiter inbetween all elements;
+ * 
+ * Return value: the gchar *
+ **/
+gchar *stringlist_to_string(GList *stringlist, gchar *delimiter) {
+	GString *strbuffer;
+	GList *tmplist;
+	strbuffer = g_string_sized_new(4096);
+	for (tmplist=g_list_first(stringlist);tmplist;tmplist=g_list_next(tmplist)) {
+		strbuffer = g_string_append(strbuffer,(char *) tmplist->data);
+		strbuffer = g_string_append(strbuffer,(char *) delimiter);
+	}
+	return g_string_free(strbuffer, FALSE);
+}
+gchar *arraylist_to_string(GList *arraylist, gchar *delimiter) {
+	GString *strbuffer;
+	GList *tmplist;
+	strbuffer = g_string_sized_new(4096);
+	for (tmplist=g_list_first(arraylist);tmplist;tmplist=g_list_next(tmplist)) {
+		gchar *tmp = array_to_string(tmplist->data);
+		strbuffer = g_string_append(strbuffer,tmp);
+		g_free(tmp);
+		strbuffer = g_string_append(strbuffer,(char *) delimiter);
+	}
+	return g_string_free(strbuffer, FALSE);	
 }
 
 gboolean put_stringlist(GFile * file, GList * which_list, gboolean is_arraylist) {
-	return put_stringlist_limited(file,which_list, -1, is_arraylist);
+	/*return put_stringlist_limited(file,which_list, -1, is_arraylist);*/
+	gchar *buf;
+	GError *gerror=NULL;
+	if (is_arraylist) {
+		buf = arraylist_to_string(which_list, "\n");
+	} else {
+		buf = stringlist_to_string(which_list, "\n");
+	}
+	g_file_replace_contents(file,buf,strlen(buf)
+				,NULL,FALSE,G_FILE_CREATE_PRIVATE,NULL,NULL,&gerror);
+	if (gerror) {
+		g_warning("save stringlist error %d %s\n",gerror->code,gerror->message);
+		g_error_free(gerror);
+		return FALSE;
+	}
+	g_free(buf);
+	return TRUE;
 }
 
-GList *remove_from_stringlist(GList *which_list, const gchar * string) {
+/*GList *remove_from_stringlist(GList *which_list, const gchar * string) {
 	if (string && strlen(string) ) {
 		GList *tmplist = g_list_first(which_list);
 		while (tmplist) {
@@ -515,9 +532,10 @@ GList *remove_from_stringlist(GList *which_list, const gchar * string) {
 	}
 	return which_list;
 }
-
+*/
 static void unlink_before(GList *tmplist) {
 	GList *prev = tmplist->prev;
+	DEBUG_MSG("unlink_before, freeing starting at %s\n",(gchar *)tmplist->data);
 	if (prev) {
 		prev->next = NULL;
 	}
@@ -542,6 +560,15 @@ GList *limit_stringlist(GList *which_list, gint num_entries, gboolean keep_end)
 	return retlist;
 }
 
+GList *find_in_stringlist(GList *thelist, const gchar *string) {
+	GList *list;
+	for (list=g_list_first(thelist);list;list=list->next) {
+		if (g_strcmp0(string, list->data)==0)
+			return list;
+	}
+	return NULL;
+} 
+
 /**
  * add_to_history_stringlist:
  * @which_list: #GList* the list to add to
@@ -558,26 +585,25 @@ GList *limit_stringlist(GList *which_list, gint num_entries, gboolean keep_end)
  * Return value: GList* with the modified list
  */
 GList *add_to_history_stringlist(GList *which_list, const gchar *string, gboolean recent_on_top, gboolean move_if_exists) {
-	if (string && strlen(string) ) {
-		GList *tmplist = g_list_first(which_list);
-		while (tmplist) {
-			if (strcmp((gchar *) tmplist->data, string) == 0) {
-				/* move this entry to the end */
-				if (move_if_exists) {
-					DEBUG_MSG("add_to_history_stringlist, entry %s exists, moving!\n", string);
-					which_list = g_list_remove_link(which_list, tmplist);
-					if (recent_on_top) {
-						return g_list_concat(tmplist, which_list);
-					} else {
-						return g_list_concat(which_list, tmplist);
-					}
-				} else {
-					return which_list;
-				}
+	GList *tmplist;
+	if (!string || string[0]=='\0' )
+		return which_list;
+	
+	tmplist = find_in_stringlist(which_list, string);
+	if (tmplist) {
+		/* move this entry to the end */
+		if (move_if_exists) {
+			DEBUG_MSG("add_to_history_stringlist, entry %s exists, moving!\n", string);
+			which_list = g_list_remove_link(which_list, tmplist);
+			if (recent_on_top) {
+				return g_list_concat(tmplist, which_list);
+			} else {
+				return g_list_concat(which_list, tmplist);
 			}
-			tmplist = g_list_next(tmplist);
+		} else {
+			return which_list;
 		}
-		/* if we arrive here the string was not yet in the list */
+	} else {
 		DEBUG_MSG("add_to_history_stringlist, adding new entry %s\n",string);
 		if (recent_on_top) {
 			which_list = g_list_prepend(which_list, g_strdup(string));
@@ -613,31 +639,6 @@ GList *add_to_stringlist(GList * which_list, const gchar * string) {
 	}
 	return which_list;
 }
-/**
- * stringlist_to_string:
- * @stringlist: a #GList * to convert
- * @delimiter: a #const gchar * item with the delimiter
- * 
- * this function will convert a stringlist (GList that contains 
- * only \0 terminated gchar* elements) to a string, putting the 
- * delimiter inbetween all elements;
- * 
- * Return value: the gchar *
- **/
-gchar *stringlist_to_string(GList *stringlist, gchar *delimiter) {
-	gchar *string, *tmp;
-	GList *tmplist;
-	string = g_strdup("");
-	tmp = string;
-	tmplist = g_list_first(stringlist);
-	while (tmplist) {
-		string = g_strconcat(tmp, (gchar *) tmplist->data, delimiter, NULL);
-		g_free(tmp);
-		tmp = string;
-		tmplist = g_list_next(tmplist);
-	}
-	return string;
-}
 
 /**
  * array_n_strings_identical:
@@ -674,6 +675,7 @@ gint array_n_strings_identical(const gchar **array1, const gchar **array2, gbool
 	}
 	return res;
 }
+#ifdef OBSOLETE
 /**
  * arraylist_delete_identical:
  * @thelist: #GList*
@@ -719,7 +721,7 @@ GList *arraylist_append_identical_from_list(GList *thelist, GList *source, const
 	while (tmplist) {
 		gchar **tmparr = tmplist->data;
 		if (array_n_strings_identical(compare, (const gchar **)tmparr, case_sensitive, testlevel)==0) {
-			thelist = g_list_append(thelist, duplicate_stringarray(tmparr));
+			thelist = g_list_append(thelist, g_strdupv(tmparr));
 		}
 		tmplist = g_list_next(tmplist);
 	}
@@ -745,7 +747,7 @@ GList *arraylist_append_identical_from_file(GList *thelist, const gchar *sourcef
 	free_arraylist(sourcelist);
 	return thelist;
 }*/
-
+#endif
 /**
  * arraylist_value_exists:
  * @arraylist: #GList*
@@ -773,6 +775,7 @@ gchar **arraylist_value_exists(GList *arraylist, const gchar **value, gint testl
 /*	g_print("arraylist_value_exists,found nothing, listlen=%d\n",g_list_length(arraylist));*/
 	return NULL;
 }
+#ifdef OBSOLETE
 /**
  * arraylist_load_new_identifiers_from_list:
  * @mylist: #GList*
@@ -791,10 +794,10 @@ GList *arraylist_load_new_identifiers_from_list(GList *mylist, GList *deflist, g
 	GList *tmplist = g_list_first(deflist);
 	while (tmplist) {
 		gchar **tmparr = tmplist->data;
-		if (count_array(tmparr) >= uniquelevel) {
+		if (g_strv_length(tmparr) >= uniquelevel) {
 			if (!arraylist_value_exists(mylist, (const gchar **)tmparr, uniquelevel, TRUE)) {
 				DEBUG_MSG("arraylist_load_new_identifiers, adding %s to thelist\n",tmparr[0]);
-				mylist = g_list_append(mylist, duplicate_stringarray(tmparr));
+				mylist = g_list_append(mylist, g_strdupv(tmparr));
 			}
 		}
 		tmplist = g_list_next(tmplist);
@@ -821,6 +824,7 @@ GList *arraylist_load_new_identifiers_from_file(GList *mylist, const gchar *from
 	free_arraylist(deflist);
 	return mylist;	
 }*/
+#endif
 
 /* pure for debugging purposes */
 #ifdef DEVELOPMENT
@@ -834,4 +838,3 @@ void dump_arraylist(GList *list) {
 }
 #endif
 
-
